/**CHeaderFile*****************************************************************

  FileName    [bmc.h]

  PackageName [bmc]

  Synopsis    [The header file for the <tt>bmc</tt> package]

  Description []

  SeeAlso     []

  Author      [Roberto Cavada]

  Copyright   [
  This file is part of the ``bmc'' package of NuSMV version 2. 
  Copyright (C) 2000-2001 by ITC-irst and University of Trento. 

  NuSMV version 2 is free software; you can redistribute it and/or 
  modify it under the terms of the GNU Lesser General Public 
  License as published by the Free Software Foundation; either 
  version 2 of the License, or (at your option) any later version.

  NuSMV version 2 is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public 
  License along with this library; if not, write to the Free Software 
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.

  For more information of NuSMV see <http://nusmv.irst.itc.it>
  or email to <nusmv-users@irst.itc.it>.
  Please report bugs to <nusmv-users@irst.itc.it>.

  To contact the NuSMV development board, email to <nusmv@irst.itc.it>. ]

******************************************************************************/

#ifndef _BMC_h
#define _BMC_h

/* Following includes do not follow standards. They were included in order 
   to make compilable this file: */
#include "rbc.h"
#include "prop.h"

#include "util.h" /* for EXTERN and ARGS */


/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

#define BMC_BEXP_OUTPUT_SMV 0
#define BMC_BEXP_OUTPUT_LB 1
#define BMC_NO_LOOP   -(INT_MAX-1) /* must be negative! */
#define BMC_ALL_LOOPS BMC_NO_LOOP+1

/* Please do not use directly following definitions. 
   Use "BMC_IS_NO_LOOPBACK" and "BMC_IS_ALL_LOOPBACKS" macros instead. 
   Function "Bmc_ConvertLoopbackFromString" can be probably more suitable than
   using these macros. */
#define BMC_ALL_LOOPS_USERSIDE_SYMBOL "*"
#define BMC_NO_LOOP_USERSIDE_SYMBOL   "X"


/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/


/* These declarations are dependant on the specific rbc package used: */
#define RBC_SPEC_TYPE         Rbc_t
#define RBC_SPEC_MANAGER_TYPE Rbc_Manager_t
#define RBC_SPEC_BOOL_TYPE    Rbc_Bool_c

/* The generic rbc manager: */
typedef RBC_SPEC_MANAGER_TYPE RbcManager;
typedef RbcManager*           RbcManager_ptr; 

/* The generic rbc structure pointer: */
typedef RBC_SPEC_TYPE*  rbc_ptr;



/*---------------------------------------------------------------------------*/
/* Structure declarations                                                    */
/*---------------------------------------------------------------------------*/

/**Struct**********************************************************************

  Synopsis    [Model Manager (core data structure)]

  Description []

  SeeAlso     []

******************************************************************************/
struct bmc_model_mgr_TAG {
  struct bmc_vars_mgr * vars_mgr; /* variables manager (reference) */
  Fsm_RbcPtr rbc_fsm;  /* The FSM in rbc format */
};

typedef struct bmc_model_mgr_TAG bmc_model_mgr;



/**Struct**********************************************************************

  Synopsis    [Bmc Problem]

  Description []

  SeeAlso     []

******************************************************************************/
struct bmc_problem {
  bmc_model_mgr * model_mgr; /* Model manager (reference) */
  node_ptr ltlspec;                 /* LTL propery */
};




/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

/**Macro***********************************************************************

  Synopsis     [Use this macro when you need to know if a string representing
  loopback value means "all possible loops"]

  Description  []

  SideEffects  []

  SeeAlso      [BMC_IS_NO_LOOPBACK]

******************************************************************************/
#define BMC_IS_ALL_LOOPBACKS(str) \
(strcmp(str, BMC_ALL_LOOPS_USERSIDE_SYMBOL) == 0)


/**Macro***********************************************************************

  Synopsis     [Use this macro when you need to know if a string representing
  loopback value means "no loopback"]

  Description  []

  SideEffects  []

  SeeAlso      [BMC_IS_ALL_LOOPBACKS]

******************************************************************************/
#define BMC_IS_NO_LOOPBACK(str) \
(strcmp(str, BMC_NO_LOOP_USERSIDE_SYMBOL) == 0)



/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/


EXTERN void Bmc_Init   ARGS((void));
EXTERN void Bmc_Setup  ARGS((void));
EXTERN void Bmc_Quit   ARGS((void));
EXTERN void Bmc_AddCmd ARGS((void));

EXTERN void Bmc_GenSolvePbs ARGS((Prop_Ptr ltlprop, 
				  const int bIncreaseK, 
				  const int bCreateDimacs, 
				  const int bSolve)); 

EXTERN void Bmc_GenSolveInvar ARGS((Prop_Ptr invarprop, 
				    const int bCreateDimacs,
				    const int bSolve));



EXTERN bmc_model_mgr * Bmc_GetModelMgr ARGS((void));
EXTERN rbc_ptr Bmc_Mk_j_k_Path ARGS((Fsm_RbcPtr rbc_fsm, int j, int k));
EXTERN rbc_ptr Bmc_Mk_k_l_Loop ARGS((Fsm_RbcPtr rbc_fsm, int k, int l));
EXTERN rbc_ptr Bmc_MkProblem_k_Path ARGS((Fsm_RbcPtr rbc_fsm, node_ptr wff, 
					  int k));

EXTERN rbc_ptr Bmc_MkProblem_k_l ARGS((Fsm_RbcPtr rbc_fsm, node_ptr wff, 
				       int k, int l));

EXTERN rbc_ptr Bmc_Mk_k_Path ARGS((Fsm_RbcPtr rbc_fsm, int k));
EXTERN rbc_ptr Bmc_MkProblem_k ARGS((Fsm_RbcPtr rbc_fsm, node_ptr wff, 
				      int k));

EXTERN rbc_ptr Bmc_MkInvariantProblem ARGS((Fsm_RbcPtr rbc_fsm, 
					    node_ptr wff));


EXTERN rbc_ptr Bmc_MkTableau_k_Noloop ARGS((Fsm_RbcPtr rbc_fsm, 
					    node_ptr ltl_wff, int k));
EXTERN rbc_ptr Bmc_MkTableau_k_l ARGS((node_ptr ltl_wff, int k, int l));
EXTERN rbc_ptr Bmc_MkTableauAtTime ARGS((node_ptr ltl_wff, 
					 int time, int k, int l));


EXTERN rbc_ptr Bmc_MkNextTableauAtTime ARGS((node_ptr wff, int time, 
					     int k, int l));

EXTERN rbc_ptr Bmc_MkEventuallyTableauAtTime ARGS((node_ptr wff, int intime,
						   int k, int l));

EXTERN rbc_ptr Bmc_MkGloballyTableauAtTime ARGS((node_ptr wff, int intime, 
						 int k, int l));

EXTERN rbc_ptr Bmc_MkUntilTableauAtTime ARGS((node_ptr p, node_ptr q, 
					      int time, int k, int l));

EXTERN rbc_ptr Bmc_MkReleasesTableauAtTime ARGS((node_ptr p, node_ptr q, 
						 int time, int k, int l));

EXTERN node_ptr Bmc_MkLoopbackLtl ARGS((int k, int l));
EXTERN node_ptr Bmc_Rbc2Bexp  ARGS((rbc_ptr rbc));



/*---------------------------------------------------------------------------*/
/* Interfaces to generic rbc                                                 */
/*---------------------------------------------------------------------------*/
 
/* Allocation/deallocation control interface. 
   (Warning! This interface should be shown carefully.) */ 
EXTERN RbcManager_ptr Bmc_RbcManagerAlloc ARGS((int varCapacity));
EXTERN void Bmc_RbcManagerReserve  ARGS((RbcManager_ptr rbcManager, 
					 int newVarCapacity));

EXTERN int  Bmc_RbcManagerCapacity ARGS((RbcManager_ptr rbcManager));
EXTERN void Bmc_RbcManagerFree     ARGS((RbcManager_ptr rbcManager));
EXTERN void Bmc_RbcManagerGC       ARGS((RbcManager_ptr rbcManager));


/* Logical operations interface: */

EXTERN int     Bmc_RbcIsTrue   ARGS((RbcManager_ptr rbcManager, rbc_ptr arg));
EXTERN int     Bmc_RbcIsFalse  ARGS((RbcManager_ptr rbcManager, rbc_ptr arg));
EXTERN rbc_ptr Bmc_RbcTruth    ARGS((RbcManager_ptr rbcManager));
EXTERN rbc_ptr Bmc_RbcFalsity  ARGS((RbcManager_ptr rbcManager));
EXTERN rbc_ptr Bmc_RbcNot      ARGS((RbcManager_ptr rbcManager, rbc_ptr arg));
EXTERN rbc_ptr Bmc_RbcAnd      ARGS((RbcManager_ptr rbcManager, rbc_ptr arg1, 
				     rbc_ptr arg2));

EXTERN rbc_ptr Bmc_RbcOr       ARGS((RbcManager_ptr rbcManager, rbc_ptr arg1, 
				     rbc_ptr arg2));

EXTERN rbc_ptr Bmc_RbcXor      ARGS((RbcManager_ptr rbcManager, rbc_ptr arg1, 
				     rbc_ptr arg2));

EXTERN rbc_ptr Bmc_RbcImplies  ARGS((RbcManager_ptr rbcManager, rbc_ptr arg1, 
				     rbc_ptr arg2));

EXTERN rbc_ptr Bmc_RbcIff      ARGS((RbcManager_ptr rbcManager, rbc_ptr arg1, 
				     rbc_ptr arg2));

EXTERN rbc_ptr Bmc_RbcIte      ARGS((RbcManager_ptr rbcManager, rbc_ptr arg_if,
				     rbc_ptr arg_then, rbc_ptr arg_else));

EXTERN rbc_ptr Bmc_RbcVarShift ARGS((RbcManager_ptr rbcManager, rbc_ptr f, 
				     int shift));

EXTERN rbc_ptr Bmc_RbcVarSubst ARGS((RbcManager_ptr rbcManager, rbc_ptr f, 
				     int* subst));

EXTERN int Bmc_RbcConvertToCnf ARGS((RbcManager_ptr rbcManager, rbc_ptr f, 
				     lsList clauses, lsList vars));

/* Utilities interface: */
EXTERN void Bmc_RbcDumpDavinci ARGS((RbcManager_ptr rbcManager, rbc_ptr f, 
				     FILE* outFile));

EXTERN void Bmc_RbcDumpSexpr   ARGS((RbcManager_ptr rbcManager, rbc_ptr f, 
				     FILE* outFile));


/* Low-level access interface: */
EXTERN rbc_ptr Bmc_RbcGetIthVar ARGS((RbcManager_ptr rbcManager, 
				      int varIndex));
EXTERN int Bmc_RbcGetVarIndex ARGS((RbcManager_ptr rbcManager, rbc_ptr var));

/* user interface useful interface: */
EXTERN outcome 
Bmc_ConvertLoopbackFromString ARGS((const char* strValue, int* iValue)); 

EXTERN void
Bmc_ConvertLoopbackFromInteger ARGS((const int iLoopback, char* szLoopback, 
				     const int bufsize));

/**AutomaticEnd***************************************************************/

#endif /* _BMC_h */

